@@ -16,23 +16,28 @@ from scipy.cluster.hierarchy import linkage as scipy_linkage
 from sklearn.cluster import KMeans, MiniBatchKMeans
 from sklearn.metrics import silhouette_score, silhouette_samples, silhouette_score
 
+
 # %% Main
-def fit(X, metric='euclidean', linkage='ward', minclusters=2, maxclusters=25, Z=None, savemem=False, verbose=3):
+def fit(X, cluster='agglomerative', metric='euclidean', linkage='ward', min_clust=2, max_clust=25, Z=None, savemem=False, verbose=3):
     """This function return the cluster labels for the optimal cutt-off based on the choosen hierarchical clustering method.
 
     Parameters
     ----------
     X : Numpy-array,
         Where rows is features and colums are samples.
+    cluster : str, (default: 'agglomerative')
+        Clustering method type for clustering.
+            * 'agglomerative'
+            * 'kmeans'
     metric : str, (default: 'euclidean').
-        Distance measure for the clustering. Options are: 'euclidean','kmeans'.
+        Distance measure for the clustering, such as 'euclidean','hamming', etc.
     linkage : str, (default: 'ward')
         Linkage type for the clustering.
         'ward','single',',complete','average','weighted','centroid','median'.
-    minclusters : int, (default: 2)
-        Number of clusters that is evaluated greater or equals to minclusters.
-    maxclusters : int, (default: 25)
-        Number of clusters that is evaluated smaller or equals to maxclusters.
+    min_clust : int, (default: 2)
+        Number of clusters that is evaluated greater or equals to min_clust.
+    max_clust : int, (default: 25)
+        Number of clusters that is evaluated smaller or equals to max_clust.
     savemem : bool, (default: False)
         Save memmory when working with large datasets. Note that htis option only in case of KMeans.
     Z : Object, (default: None).
@@ -84,37 +89,36 @@ def fit(X, metric='euclidean', linkage='ward', minclusters=2, maxclusters=25, Z=
     # Make dictionary to store Parameters
     Param = {}
     Param['verbose'] = verbose
+    Param['cluster'] = cluster
     Param['metric'] = metric
     Param['linkage'] = linkage
-    Param['minclusters'] = minclusters
-    Param['maxclusters'] = maxclusters
+    Param['min_clust'] = min_clust
+    Param['max_clust'] = max_clust
     Param['savemem'] = savemem
+    if verbose>=3: print('[clusteval] >Evaluate using silhouette.')
 
     # Savemem for kmeans
-    if Param['metric']=='kmeans':
+    if Param['cluster']=='kmeans':
         if Param['savemem']:
             kmeansmodel = MiniBatchKMeans
-            if Param['verbose']>=3: print('[silhouette] >Save memory enabled for kmeans with method silhouette.')
+            if Param['verbose']>=3: print('[clusteval] >Save memory enabled for kmeans with method silhouette.')
         else:
             kmeansmodel = KMeans
 
     # Cluster hierarchical using on metric/linkage
-    if (Z is None) and (Param['metric']!='kmeans'):
+    if (Z is None) and (Param['cluster']!='kmeans'):
         Z = scipy_linkage(X, method=Param['linkage'], metric=Param['metric'])
 
-    # Make all possible cluster-cut-offs
-    if Param['verbose']>=3: print('[silhouette] >Determining optimal [%s] clustering by silhouette score..' %(Param['metric']))
-
     # Setup storing parameters
-    clustcutt = np.arange(Param['minclusters'],Param['maxclusters'])
-    silscores = np.zeros((len(clustcutt)))*np.nan
-    sillclust = np.zeros((len(clustcutt)))*np.nan
+    clustcutt = np.arange(Param['min_clust'], Param['max_clust'])
+    silscores = np.zeros((len(clustcutt))) * np.nan
+    sillclust = np.zeros((len(clustcutt))) * np.nan
     clustlabx = []
 
     # Run over all cluster cutoffs
     for i in tqdm(range(len(clustcutt))):
         # Cut the dendrogram for i clusters
-        if Param['metric']=='kmeans':
+        if Param['cluster']=='kmeans':
             labx = kmeansmodel(n_clusters=clustcutt[i], verbose=0).fit(X).labels_
         else:
             labx = fcluster(Z, clustcutt[i], criterion='maxclust')
@@ -132,23 +136,34 @@ def fit(X, metric='euclidean', linkage='ward', minclusters=2, maxclusters=25, Z=
 
     # Store only if agrees to restriction of input clusters number
     I1 = np.isnan(silscores)==False
-    I2 = sillclust>=Param['minclusters']
-    I3 = sillclust<=Param['maxclusters']
-    I  = I1 & I2 & I3
+    I2 = sillclust>=Param['min_clust']
+    I3 = sillclust<=Param['max_clust']
+    Iloc = I1 & I2 & I3
 
-    # Get only clusters of interest
-    silscores = silscores[I]
-    sillclust = sillclust[I]
-    clustlabx = clustlabx[I,:]
-    clustcutt = clustcutt[I]
-    idx = np.argmax(silscores)
+    if verbose>=5:
+        print(clustlabx)
+        print('Iloc: %s' %(str(Iloc)))
+        print('silscores: %s' %(str(silscores)))
+        print('sillclust: %s' %(str(sillclust)))
+        print('clustlabx: %s' %(str(clustlabx)))
+
+    if len(Iloc)>0:
+        # Get only clusters of interest
+        silscores = silscores[Iloc]
+        sillclust = sillclust[Iloc]
+        clustlabx = clustlabx[Iloc, :]
+        clustcutt = clustcutt[Iloc]
+        idx = np.argmax(silscores)
+        clustlabx = clustlabx[idx, :] - 1
+    else:
+        if verbose>=3: print('[clusteval] >No clusters detected.')
 
     # Store results
     results = {}
     results['method']='silhouette'
-    results['score'] = pd.DataFrame(np.array([sillclust,silscores]).T, columns=['clusters','score'])
+    results['score'] = pd.DataFrame(np.array([sillclust, silscores]).T, columns=['clusters', 'score'])
     results['score']['clusters'] = results['score']['clusters'].astype(int)
-    results['labx']  = clustlabx[idx,:]-1
+    results['labx'] = clustlabx
     results['fig'] = {}
     results['fig']['silscores'] = silscores
     results['fig']['sillclust'] = sillclust
@@ -180,7 +195,7 @@ def plot(results, figsize=(15,8)):
     # Plot
     ax1.plot(results['fig']['sillclust'], results['fig']['silscores'], color='k')
     # Plot optimal cut
-    ax1.axvline(x=results['fig']['clustcutt'][idx], ymin=0, ymax=results['fig']['sillclust'][idx], linewidth=2, color='r',linestyle="--")
+    ax1.axvline(x=results['fig']['clustcutt'][idx], ymin=0, ymax=results['fig']['sillclust'][idx], linewidth=2, color='r', linestyle="--")
     # Set fontsizes
     plt.rc('axes', titlesize=14)  # fontsize of the axes title
     plt.rc('xtick', labelsize=10)  # fontsize of the axes title
@@ -194,11 +209,11 @@ def plot(results, figsize=(15,8)):
     ax1.grid(color='grey', linestyle='--', linewidth=0.2)
     plt.show()
     # Return
-    return(fig,ax1)
+    return(fig, ax1)
 
 
 # %% Scatter data
-def scatter(labx, X=None, figsize=(15,8), verbose=3):
+def scatter(labx, X=None, figsize=(15, 8), verbose=3):
     """Make scatter for the cluster labels with the samples.
 
     Parameters
@@ -217,7 +232,7 @@ def scatter(labx, X=None, figsize=(15,8), verbose=3):
 
     """
     if X is None:
-        if verbose>=2: print('[silhouette] >Warning: Input data X is required for the scatterplot.')
+        if verbose>=2: print('[clusteval] >Warning: Input data X is required for the scatterplot.')
         return None
 
     # Label
@@ -225,14 +240,14 @@ def scatter(labx, X=None, figsize=(15,8), verbose=3):
         labx = labx.get('labx', None)
     # Check labx
     if labx is None:
-        if verbose>=3: print('[silhouette] >Error: No labels provided.')
+        if verbose>=3: print('[clusteval] >Error: No labels provided.')
         return None
 
     # Plot silhouette samples plot
     # n_clusters = len(np.unique(labx))
     n_clusters = len(set(labx)) - (1 if -1 in labx else 0)
     silhouette_avg = silhouette_score(X, labx)
-    if verbose>=3: print('[silhouette] >Estimated number of n_clusters: %d, average silhouette_score=%.3f' %(n_clusters, silhouette_avg))
+    if verbose>=3: print('[clusteval] >Estimated number of n_clusters: %d, average silhouette_score=%.3f' %(n_clusters, silhouette_avg))
 
     # Compute the silhouette scores for each sample
     sample_silhouette_values = silhouette_samples(X, labx)
@@ -248,7 +263,7 @@ def scatter(labx, X=None, figsize=(15,8), verbose=3):
     uiclust = np.unique(labx)
 
     # Make 1st plot
-    for i in range(0,len(uiclust)):
+    for i in range(0, len(uiclust)):
         # Aggregate the silhouette scores for samples belonging to
         # cluster i, and sort them
         ith_cluster_silhouette_values = sample_silhouette_values[labx == uiclust[i]]
@@ -276,7 +291,7 @@ def scatter(labx, X=None, figsize=(15,8), verbose=3):
 
     # 2nd Plot showing the actual clusters formed
     color = cm.Set2(labx.astype(float) / n_clusters)
-    ax2.scatter(X[:, 0], X[:, 1], marker='.', s=30, lw=0, alpha=0.8,c=color, edgecolor='k')
+    ax2.scatter(X[:, 0], X[:, 1], marker='.', s=30, lw=0, alpha=0.8, c=color, edgecolor='k')
     ax2.grid(color='grey', linestyle='--', linewidth=0.2)
     ax2.set_title("Estimated cluster labels")
     ax2.set_xlabel("1st feature")
